'\" Copyright 1989 Regents of the University of California
'\" Permission to use, copy, modify, and distribute this
'\" documentation for any purpose and without fee is hereby
'\" granted, provided that this notice appears in all copies.
'\" The University of California makes no representations about
'\" the suitability of this material for any purpose.  It is
'\" provided "as is" without express or implied warranty.
'\" 
'\" $Header: /sprite/src/lib/tcl/RCS/Tcl_AddErrorInfo.man,v 1.2 90/01/15 14:58:47 ouster Exp $ SPRITE (Berkeley)
'\" 
.\"vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
.\" These macros make this man page compatible with the standard manual files.
.\" They are replacement for the standard sprite macros, the results are not
.\" as beautiful, but they work with standard nroff.
.\"
.\" .HS name section [date [version]]
.\"	Replacement for .TH in other man pages.
.\"
.\" .LG
.\"	Increase font size;  opposite of .SM
.\"
.\" .AP type name in/out [indent]
.\"	Start paragraph describing an argument to a library procedure.
.\"	type is type of argument (int, etc.), in/out is either "in", "out",
.\"	or "in/out" to describe whether procedure reads or modifies arg,
.\"	and indent is equivalent to second arg of .IP (shouldn't ever be
.\"	needed;  use .AS below instead)
.\"
.\" .AS [type [name]]
.\"	Give maximum sizes of arguments for setting tab stops.  Type and
.\"	name are examples of largest possible arguments that will be passed
.\"	to .AP later.  If args are omitted, default tab stops are used.
.\"
'@header
'	# Heading for Sprite man pages
.de HS
.TH "\\$1" "TCL"
..
'	# Increase point size 1 tick
.de LG
.ps +1
.it 1 }N
.if !"\\$1"" \&\\$1 \\$2 \\$3 \\$4 \\$5 \\$6
..
'	# Start an argument description
.de AP
.ie !"\\$4"" .TP \\$4
.el \{\
.   ie !"\\$2"" .TP \\n()Cu
.   el          .TP 15
.\}
.ie !"\\$3"" \{\
.ta \\n()Au \\n()Bu
\&\\$1	\\fI\\$2\\fP	(\\$3)
.\".b
.\}
.el \{\
.br
.ie !"\\$2"" \{\
\&\\$1	\\fI\\$2\\fP
.\}
.el \{\
\&\\fI\\$1\\fP
.\}
.\}
.DT
..
'	# define tabbing values for .AP
.de AS
.nr )A 10n
.if !"\\$1"" .nr )A \\w'\\$1'u+3n
.nr )B \\n()Au+15n
.\"
.if !"\\$2"" .nr )B \\w'\\$2'u+\\n()Au+3n
.nr )C \\n()Bu+\\w'(in/out)'u+2n
..
'	# BS - start boxed text
.de BS
' no-op
..
'	# BE - end boxed text (draw box now)
.de BE
' no-op
..
'	# VS - start vertical sidebar
.de VS
' no-op
..
'	# VE - end of vertical sidebar
.de VE
' no-op
..
'       # DS - start somekind of indentation
.de DS
.sp
.nf
'       # DE - end somekind of indentation
..
.de DE
.sp
.fi
..
'@endheader
.\"^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
.HS Tcl_AddErrorInfo tcl
.BS
.SH NAME
Tcl_AddErrorInfo \- record information about where an error occurred
.SH SYNOPSIS
.nf
\fB#include <tcl.h>\fR
.sp
char *
\fBTcl_AddErrorInfo\fR(\fIinterp, format, arg ...\fR)
.SH ARGUMENTS
.AS Tcl_Interp *interp
.AP Tcl_Interp *interp in
Interpreter in which to record information.
.AP char *format in
Sprintf style format string to format the remaining arguments.  The resulting
string must not exceed \fBTCL_RESULT_SIZE\fR.
.BE

.SH DESCRIPTION
.PP
Whenever \fBTcl_Eval\fR returns a \fBTCL_ERROR\fR return code,
it also places information in a global variable \fBerrorInfo\fR
that describes the command that was being executed when the error
occurred.  If multiple commands or procedures were nested when
the error occurred, additional information gets added to
\fBerrorInfo\fR as the \fBTCL_ERROR\fR code is passed upwards.
Eventually, \fBerrorInfo\fR will give a complete trace
of the activity that was in progress when the error occurred.
.PP
The procedure \fBTcl_AddErrorInfo\fR may be invoked to record
additional information about what was happening when an error
occurred.  For example, the \fBsource\fR command uses this
procedure to record the name of the file being processed and the
line number on which the error occurred;  for Tcl procedures, the
procedure name and line number within the procedure are recorded,
and so on.  The extra information is contained in the \fImessage\fR
argument, and is normally enclosed in parentheses.  If \fBTcl_AddErrorInfo\fR
is to be called, it should be called by code that invokes
\fBTcl_Eval\fR;  \fBTcl_AddErrorInfo\fR should be called
just after \fBTcl_Eval\fR returns \fBTCL_ERROR\fR.  In calling
\fBTcl_AddErrorInfo\fR, you may find it useful to use the \fBerrorLine\fR
field of the interpreter (see the \fBTcl_Interp\fR manual entry for
details).

.SH KEYWORDS
error, stack, trace, variable
