'\" Copyright 1989 Regents of the University of California
'\" Permission to use, copy, modify, and distribute this
'\" documentation for any purpose and without fee is hereby
'\" granted, provided that this notice appears in all copies.
'\" The University of California makes no representations about
'\" the suitability of this material for any purpose.  It is
'\" provided "as is" without express or implied warranty.
'\" 
'\" $Header:
'\" 
.\"vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
.\" These macros make this man page compatible with the standard manual files.
.\" They are replacement for the standard sprite macros, the results are not
.\" as beautiful, but they work with standard nroff.
.\"
.\" .HS name section [date [version]]
.\"	Replacement for .TH in other man pages.
.\"
.\" .LG
.\"	Increase font size;  opposite of .SM
.\"
.\" .AP type name in/out [indent]
.\"	Start paragraph describing an argument to a library procedure.
.\"	type is type of argument (int, etc.), in/out is either "in", "out",
.\"	or "in/out" to describe whether procedure reads or modifies arg,
.\"	and indent is equivalent to second arg of .IP (shouldn't ever be
.\"	needed;  use .AS below instead)
.\"
.\" .AS [type [name]]
.\"	Give maximum sizes of arguments for setting tab stops.  Type and
.\"	name are examples of largest possible arguments that will be passed
.\"	to .AP later.  If args are omitted, default tab stops are used.
.\"
'@header
'	# Heading for Sprite man pages
.de HS
.TH "\\$1" "TCL"
..
'	# Increase point size 1 tick
.de LG
.ps +1
.it 1 }N
.if !"\\$1"" \&\\$1 \\$2 \\$3 \\$4 \\$5 \\$6
..
'	# Start an argument description
.de AP
.ie !"\\$4"" .TP \\$4
.el \{\
.   ie !"\\$2"" .TP \\n()Cu
.   el          .TP 15
.\}
.ie !"\\$3"" \{\
.ta \\n()Au \\n()Bu
\&\\$1	\\fI\\$2\\fP	(\\$3)
.\".b
.\}
.el \{\
.br
.ie !"\\$2"" \{\
\&\\$1	\\fI\\$2\\fP
.\}
.el \{\
\&\\fI\\$1\\fP
.\}
.\}
.DT
..
'	# define tabbing values for .AP
.de AS
.nr )A 10n
.if !"\\$1"" .nr )A \\w'\\$1'u+3n
.nr )B \\n()Au+15n
.\"
.if !"\\$2"" .nr )B \\w'\\$2'u+\\n()Au+3n
.nr )C \\n()Bu+\\w'(in/out)'u+2n
..
'	# BS - start boxed text
.de BS
' no-op
..
'	# BE - end boxed text (draw box now)
.de BE
' no-op
..
'	# VS - start vertical sidebar
.de VS
' no-op
..
'	# VE - end of vertical sidebar
.de VE
' no-op
..
'       # DS - start somekind of indentation
.de DS
.sp
.nf
'       # DE - end somekind of indentation
..
.de DE
.sp
.fi
..
'@endheader
.\"^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
.HS Tcl_RetError tcl
.BS
.SH NAME
Tcl_RetError \- format an error message as a command result.
.SH SYNOPSIS
.nf
\fB#include <tcl.h>\fR
.sp
char *
\fBTcl_RetError\fR(\fIinterp, errorCode, format, arg ...\fR)
.SH ARGUMENTS
.AS Tcl_Interp *errorCode
.AP Tcl_Interp *interp in
Interpreter in which to record information.
.AP char *errorCode in
List to set as the value of the errorCode variable.  If NULL is specified,
then the errorCode is left unchanged.
.AP char *format in
Sprintf style format string to format the remaining arguments.  The resulting
string must not exceed \fBTCL_RESULT_SIZE\fR.
.BE

.SH DESCRIPTION
.PP
This procedure is an easy way to format an error message as the result of a 
command.  A sprintf style of formatting is used to encode a variable number of
arguments into the result.  The errorCode variable may optionally be set.  If
a complex error code must be formatted, use Tcl_SetErrorCode and specify NULL
for the errorCode parameter.  This function always returns the value TCL_ERROR,
so this can be used as a one-line set error code, numeric and string results.
For example:
.sp
.nf
    if (argc != 1)
	return Tcl_RetError (interp, "TCL", 
             "too many args: should be \e"%.50s\e"", argv[0]);
.fi

.SH KEYWORDS
error
