/* 
 * tclUtil. --
 *
 *      Contains a macro user to support variable argument format strings.
 *
 */

/* 
 * Copyright 1990 Karl Lehenbauer, Mark Diekhans, 
 *                Peter da Silva and Jordan Henderson.
 *
 * Permission to use, copy, modify, and distribute this
 * software and its documentation for any purpose and without
 * fee is hereby granted, provided that the above copyright
 * notice appear in all copies.  Karl Lehenbauer, Mark Diekhans,
 * Peter da Silva and Jordan Henderson make no representations
 * about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 */


/*
 *----------------------------------------------------------------------
 *
 * GET_FORMATTED_LEN --
 *     This macro is used to determine the length of a resulting sprintf
 *     based on the format spec and varargs to be encoded.  The args may
 *     only be strings, since this makes it easiest to determine the 
 *     length.  This is implemented as a macro, since there is no portable
 *     method of passing varargs on from one procedure to another.  This
 *     is not desirable, but :-(.  Assumes that there is a va_alist 
 *     parameter declared in the referencing procedure.
 *
 * Parameters:
 *     formatStr - Sprintf style format string.
 *     resultLen - The resulting length of the format string, include 
 *       the NULL.
 *     
 *----------------------------------------------------------------------
 */
#ifndef XENIX286
#define GET_FORMATTED_LEN(formatStr, resultLen) \
{ \
char    *scanPtr = formatStr; \
int      numArgs = 0; \
char    *argPtr; \
va_list  args; \
\
resultLen = strlen (scanPtr) + 1; \
for (; *scanPtr != '\0'; scanPtr++) \
    if ((*scanPtr == '%') && (*(++scanPtr) != '%')) \
        numArgs++; \
va_start (args); \
for (; numArgs > 0; numArgs--) { \
    argPtr = va_arg (args, char*); \
    resultLen += strlen (argPtr);} \
va_end (args); \
}
#else
/*
 * Xenix 286 C compiler is a real piece of sh*t.  Here is a shrunk-down
 * version of the GET_FORMATTED_LEN to keep it from barfing.
 *
 * fS = formatStr
 * rL = resultLen
 * sP = scanPtr
 * nA = numArgs
 * aP = argPtr
 */
#define GET_FORMATTED_LEN(fS,rL) {\
char *sP = fS;\
int nA = 0;\
char *aP;\
va_list args;\
rL = strlen(sP)+1;\
for(;*sP!='\0';sP++)\
 if((*sP=='%')&&(*(++sP)!='%'))nA++;\
va_start(args);\
for(;nA>0;nA--){\
 aP=va_arg(args,char*);\
 rL+=strlen(aP);}\
va_end(args);\
}
#endif
