."\ Portions of this manual page are:
."\    Copyright (c) 1986 by University of Toronto.
."\    Written by Henry Spencer.  Not derived from licensed software.
.TH REGEXP TCL "" "TCL - Tool Command Language"
.ad b
.SH NAME
regexp, regsub - TCL regular expression comparison and substitution commands.
.SH SYNOPSIS
.br
.B regexp \fIexp string\fR [\fBnocase\fR] [\fImatchvar\fR] 
[\fIsubmatchvar1 .. submatchvar9\fR]
.br
.B regsub \fIexp string subspec retvar\fR [\fBnocase\fR]
.br
.SH DESCRIPTION
.PP
'@help: extend/regexp
.B regexp \fIexp string\fR [\fBnocase\fR] [\fImatchvar\fR] 
[\fIsubmatchvar1 .. submatchvar9\fR]
.br
.P
Test if \fIstring\fR matches the regular expression \fIexp\fR.  If \fBnocase\fR
is specified, then the match is done regardless of case.  If \fImatchvar\fR is
specified, a list is return in the variable which has the starting index and
length of the expression that was matched in the \fIstring\fR.  This is useful
for extracting the matched expression from \fIstring\fR using \fBcrange\fR.
The \fIsubmatchvarN\fR variables are used to return the index and length of
matched subexpressions using the "(...)" syntax. A null string, {}, may be
specified for either \fImatchvar\fR or the \fIsubmatchvarN\fR parameters if
the value is not to be returned, but following arguments need to be specified.
'@:.br
'@:See also: regexp and regexpressions
'@endhelp
.PP
'@help: extend/regsub
.B regsub \fIexp string subspec retvar\fR [\fBnocase\fR]
.br
This command matches a regular expression against a string as with the
\fBregexp\fR command.  It returns 1 if the match succeeds and 0 if it fails.
If \fBnocase\fR is specified, the match is done regardless of case.
If the match succeeds, then \fIstring\fR will be copied to the variable 
\fIretvar\fR with the matched part of \fIstring\fR being substituted by
a string  specified by \fIsubspec\fR.  A `&' or a `\e0' in \fIsubspec\fR
is substituted with the entire matched string. A `\en', where n is a digit
between 1 and 9 will be substituted with the n-th matching sub-expression, 
as was delimited by `(...)' in \fBregexp\fR.  All other characters in 
\fIsubspec\fR are substituted as is.  The substitution specification should be
quoted with `{...}', to prevent the interpretation of the `\en' specifications
as special characters.  If special characters must be specified in the string,
then the `\en' sequences must be escaped and `{...}' not used.  For example:
.sp
.nf
    regsub {foo(.*)baz} $buffer {foo\e1} outbuf
    regsub {foo(.*)baz} $buffer "foo\e\e1\et" outbuf
.fi
'@:.br
'@:See also: regexp and regexpressions
'@endhelp
'@help: extend/regexpressions
.SH "REGULAR EXPRESSION SYNTAX"
.P
The \fBregexp\fR command is implemented using a modified version of Henry
Spencer's regcomp and regexec routines functions and implements their full
regular expression syntax (the same as the \fBegrep\fR).  If the expression
contains no meta-characters, then Boyer-Moore string searching is used.
.P
A regular expression is zero or more \fIbranches\fR, separated by `|'.
It matches anything that matches one of the branches.
.PP
A branch is zero or more \fIpieces\fR, concatenated.
It matches a match for the first, followed by a match for the second, etc.
.PP
A piece is an \fIatom\fR possibly followed by `*', `+', or `?'.
An atom followed by `*' matches a sequence of 0 or more matches of the atom.
An atom followed by `+' matches a sequence of 1 or more matches of the atom.
An atom followed by `?' matches a match of the atom, or the null string.
.PP
An atom is a regular expression in parentheses (matching a match for the
regular expression), a \fIrange\fR (see below), `.'
(matching any single character), `^' (matching the null string at the
beginning of the input string), `$' (matching the null string at the
end of the input string), a `\e' followed by a single character (matching
that character), or a single character with no other significance
(matching that character).
.PP
A \fIrange\fR is a sequence of characters enclosed in `[]'.
It normally matches any single character from the sequence.
If the sequence begins with `^',
it matches any single character \fInot\fR from the rest of the sequence.
If two characters in the sequence are separated by `\-', this is shorthand
for the full list of ASCII characters between them
(e.g. `[0-9]' matches any decimal digit).
To include a literal `]' in the sequence, make it the first character
(following a possible `^').
To include a literal `\-', make it the first or last character.
.SH AMBIGUITY
If a regular expression could match two different parts of the input string,
it will match the one which begins earliest.
If both begin in the same place	but match different lengths, or match
the same length in different ways, life gets messier, as follows.
.PP
In general, the possibilities in a list of branches are considered in
left-to-right order, the possibilities for `*', `+', and `?' are
considered longest-first, nested constructs are considered from the
outermost in, and concatenated constructs are considered leftmost-first.
The match that will be chosen is the one that uses the earliest
possibility in the first choice that has to be made.
If there is more than one choice, the next will be made in the same manner
(earliest possibility) subject to the decision on the first choice.
And so forth.
.PP
For example, `(ab|a)b*c' could match `abc' in one of two ways.
The first choice is between `ab' and `a'; since `ab' is earlier, and does
lead to a successful overall match, it is chosen.
Since the `b' is already spoken for,
the `b*' must match its last possibility\(emthe empty string\(emsince
it must respect the earlier choice.
.PP
In the particular case where no `|'s are present and there is only one
`*', `+', or `?', the net effect is that the longest possible
match will be chosen.
So `ab*', presented with `xabbbby', will match `abbbb'.
Note that if `ab*' is tried against `xabyabbbz', it
will match `ab' just after `x', due to the begins-earliest rule.
(In effect, the decision on where to start the match is the first choice
to be made, hence subsequent choices must respect it even if this leads them
to less-preferred alternatives.)
'
.SH "META CHARACTERS AND TCL"
Some of the regular expression characters, such as `[' and `]' also have
special meaning to the TCL interpreter.  The best approach is to quote
regular expressions with `{...}'.  The only draw back to this approach is that
normal TCL backslash substitution to insert special characters does not work,
in order to get these characters you need to escape the expression 
meta-characters and not use `{...}'.  For example:
.sp
.nf
    regexp {[a-z].*$} $buffer
    regexp "\e[a-z\e]\et" $buffer
.fi
'@endhelp
.SH INITIALIZATION
To initialize the regexp command in a custom TCL environment call:
.sp
.nf
.ft CW
    void Tcl_InitRegExp (interp)
           Tcl_Interp *interp;
.ft R
.fi
.SH BUGS
.PP
If \fBnocase\fR is specified uppercase character ranges, such as [A-Z] will
be converted to lower case, i.e. [a-z].  
.PP
Regexp should understand the Tcl special backslash characters, such as \et,
so that it can expand them and all regular expressions may be protected with
"{...}".
