/* 
 * Copyright 1990 Karl Lehenbauer, Mark Diekhans, 
 *                Peter da Silva and Jordan Henderson.
 *
 * Permission to use, copy, modify, and distribute this
 * software and its documentation for any purpose and without
 * fee is hereby granted, provided that the above copyright
 * notice appear in all copies.  Karl Lehenbauer, Mark Diekhans,
 * Peter da Silva and Jordan Henderson make no representations
 * about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 */


#ifndef NOSCCSID
        static char *sccs_id = "%M% %I% %H%";
#endif

#include <stdio.h>

/* %&#W$* Xenix 286 sys/param.h includes sys/types.h */
#ifndef XENIX286
#  include <sys/types.h>
#endif
#include <sys/param.h>
#include <time.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "tcl.h"

extern int errno;

extern char *getenv();
extern char *getcwd();
extern char *ctime();

#ifdef BSD
#  define TCL_MAX_PATH MAXPATHLEN
#else
#  define TCL_MAX_PATH 256
#endif

/*
 * A current directory cache is kept by pwd command.  This flag indicates that
 * in may be invalid because of a CD.
 */
static int pwdCacheValid = FALSE;


/*
 *----------------------------------------------------------------------
 *
 * CmdCd --
 *     Implements the TCL cd command:
 *         cd [directory]
 *     See the oscmds(TCL) manual page.
 *
 * Results:
 *      Standard TCL results, may return the UNIX system error message.
 *
 *----------------------------------------------------------------------
 */
int
CmdCd (clientData, interp, argc, argv)
    char       *clientData;
    Tcl_Interp *interp;
    int         argc;
    char      **argv;
{
    int   i;
    char *newDir;

    if (argc > 2)
        return Tcl_RetError (interp, "TCL", "bad # arg: %.50s dir", argv[0]);

    if (argc == 1)
        {
        newDir = getenv("HOME");
        if (newDir == NULL)
                return Tcl_RetError (interp, "TCL", 
                                     "no HOME in environment: cd");
        }
    else
        newDir = argv[1]; 
    if (newDir[0] == '~')
        if ((newDir = Tcl_TildeSubst (interp, newDir)) == NULL)
            return TCL_ERROR;

    if (chdir (newDir) < 0)
        return Tcl_RetSysError (interp, "%s", newDir);

    pwdCacheValid = FALSE;
    return TCL_OK;
}

/*
 *----------------------------------------------------------------------
 *
 * CmdPwd --
 *     Implements the TCL pwd command:
 *         pwd
 *     See the oscmds(TCL) manual page.
 *
 * Results:
 *      Standard TCL results, may return the UNIX system error message.
 *
 *----------------------------------------------------------------------
 */
int
CmdPwd (clientData, interp, argc, argv)
    char       *clientData;
    Tcl_Interp *interp;
    int         argc;
    char      **argv;
{
    static char pwdCache [TCL_MAX_PATH+1];
    static int  pwdCacheLen;

    if (argc != 1)
        return Tcl_RetError (interp, "TCL", "bad # arg: %.50s", argv[0]);
    if (!pwdCacheValid)
        { /* Cache not valid */
#ifndef BSD                                     
        if (getcwd (pwdCache, TCL_MAX_PATH) == NULL)
            return Tcl_RetError (interp, 
                                 "TCL", "pwd: path length exceeds max", 
                                 argv[0]);
#else
        getwd (pwdCache); 
#endif
        pwdCacheLen = strlen (pwdCache);
        pwdCacheValid = TRUE;
        } /* Cache not valid */

    if (pwdCacheLen >= TCL_RESULT_SIZE) {
        interp->result = ckalloc (pwdCacheLen+1);
        interp->dynamic = TCL_DYNAMIC;
    }
    strcpy (interp->result, pwdCache);
    return TCL_OK;
}

/*
 *----------------------------------------------------------------------
 *
 * CmdSleep --
 *     Implements the TCL sleep command:
 *         sleep seconds
 *     See the oscmds(TCL) manual page.
 *
 * Results:
 *      Standard TCL results, may return the UNIX system error message.
 *
 *----------------------------------------------------------------------
 */
int
CmdSleep (clientData, interp, argc, argv)
    ClientData  clientData;
    Tcl_Interp *interp;
    int         argc;
    char      **argv;
{
int count;

if (argc != 2)
    return Tcl_RetError (interp, "TCL", "bad # args: %.50s seconds", argv[0]);

if (!StrToInt (argv[1], 10, &count))
    return Tcl_RetError (interp, "TCL", "%.50s: Invalid numeric argument", 
                         argv[0]);

sleep(count);
return TCL_OK;
}

/*
 *----------------------------------------------------------------------
 *
 * Tcl_InitOsCmds --
 *      Initialize general OS access commands and install them into Tcl
 *
 * Side effects:
 *      The OS access commands are installed into the specified
 *      interpreter.
 *
 *----------------------------------------------------------------------
 */
Tcl_InitOsCmds (interp)
Tcl_Interp *interp;
{
Tcl_CreateCommand (interp, "sleep", CmdSleep, (ClientData)NULL, 
                  (void (*)())NULL);
Tcl_CreateCommand (interp, "cd", CmdCd, (ClientData)NULL,
                  (void (*)())NULL);
Tcl_CreateCommand (interp, "pwd", CmdPwd, (ClientData)NULL,
                  (void (*)())NULL);
}




