/* 
 * Copyright 1990 Karl Lehenbauer, Mark Diekhans, 
 *                Peter da Silva and Jordan Henderson.
 *
 * Permission to use, copy, modify, and distribute this
 * software and its documentation for any purpose and without
 * fee is hereby granted, provided that the above copyright
 * notice appear in all copies.  Karl Lehenbauer, Mark Diekhans,
 * Peter da Silva and Jordan Henderson make no representations
 * about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 */

#ifndef NOSCCSID
        static char *sccs_id = "%M% %I% %H%";
#endif

#include <stdio.h>
#include <errno.h>

#include "tcl.h"

#define MAX_EXPANSION 255


/*
 *----------------------------------------------------------------------
 *
 * ExpandString --
 *      Build an expand version of a translit range specification.
 *
 * Results:
 *      TRUE it the expansion is ok, FALSE it its too long.
 *
 *----------------------------------------------------------------------
 */
static unsigned int
ExpandString (s, buf)
    unsigned char *s;
    unsigned char  buf[];
{
        int i, j;

        i = 0;
        while((*s !=0) && i < MAX_EXPANSION) {
                if(s[1] == '-' && s[2] > s[0]) {
                        for(j = s[0]; j <= s[2]; j++)
                                buf[i++] = j;
                        s += 3;
                }
                else
                        buf[i++] = *s++;
        }
        buf[i] = 0;
        return (i < MAX_EXPANSION);
}

/*
 *----------------------------------------------------------------------
 *
 * CmdTranslit --
 *     Implements the TCL translit command:
 *         translit inrange outrange string
 *     See the string(TCL) manual page.
 *
 * Results:
 *      Standard TCL results.
 *
 *----------------------------------------------------------------------
 */
int
CmdTranslit (clientData, interp, argc, argv)
    char       *clientData;
    Tcl_Interp *interp;
    int         argc;
    char       **argv;
{
        unsigned char from[MAX_EXPANSION+1];
        unsigned char to[MAX_EXPANSION+1];
        unsigned char map[MAX_EXPANSION+1];
        unsigned char *s, *t;
        int i;

        if (argc != 4)
                return Tcl_RetError (interp, "TCL", 
                                     "bad # args: %s from to string", argv[0]);

        if (!ExpandString (argv[1], from))
                return Tcl_RetError (interp, "TCL", 
                                     "inrange expansion too long");

        if (!ExpandString (argv[2], to))
                return Tcl_RetError (interp, "TCL", 
                                     "outrange expansion too long");

        for(i = 0; i <= MAX_EXPANSION ; i++)
                map[i] = i;

        for(i = 0; to[i] != 0; i++)
                if(from[i])
                        map[from[i]] = to[i];
                else
                        break;
        if(to[i] != 0)
                return Tcl_RetError (interp, "TCL", 
                                     "inrange longer than outrange");

        for(; from[i]; i++)
                map[from[i]] = 0;

        for (s = t = (unsigned char *)argv[3]; *s; s++) {
                if(map[*s])
                        *t++ = map[*s];
        }
        *t = 0;

        Tcl_Return(interp, argv[3], TCL_VOLATILE);

        return TCL_OK;
}

/*
 *----------------------------------------------------------------------
 *
 * Tcl_InitTransLit --
 *      Initialize translit command and install it into Tcl,
 *
 *----------------------------------------------------------------------
 */
Tcl_InitTransLit(interp)
    Tcl_Interp *interp;
{
Tcl_CreateCommand (interp, "translit", CmdTranslit, (ClientData)NULL,
                  (void (*)())NULL);
}

