.TH PACKAGES TCL "" "TCL - Tool Command Language"
.ad b
.SH RATIONALE

If Tcl catches on at a lot of sites, many sites will end up with a
lot of Tcl code.
.PP
Since Tcl functions work on such a simple-but-powerful set of data types
(strings and lists), there are a lot of opportunities for code reuse.
.PP
Bugs crop up and revisions become a lot more difficult when you have to
cut and paste common routines into your various packages.  If you find
a bug in the "mkpath" function, you may have to fix it in several places,
and of course it can be difficult to ensure that you have actually
found and fixed every instance of the copied code.
.PP
To make it easier for Tcl programmers to reuse code, \fIautoloads\fR and
\fIpackages\fR are provided.
.PP
.SH AUTOLOAD

The \fIautoload\fR procedure, defined in \fBTclInit.tcl\fR, allows the
Tcl programmer
to name one or more functions and the file they are defined in.  After
\fIautoload\fR has been executed to inform Tcl of the presence of them,
a subsequent
attempt to execute one will cause the file to be loaded transparently
prior to executing the function.
For example,
.PP
	\fBautoload showproc.tcl showproc showprocs\fR
.PP
...tells Tcl to automatically load the file \fBshowproc.tcl\fR if
an attempt is made to execute \fIshowproc\fR or \fIshowprocs\fR
and the requested procedure doesn't exist.
.PP
\fIAutoload\fR is simpler but less capable than \fIpackage\fR.  
Packages libraries have the advantages that they can contain
multiple packages in one file and that no code has to be executed
in the session environment to make the demand-loading of packages
work.
.PP

.SH PACKAGES

Packages work like \fIautoload\fR, except that a package file can contain
multiple independent Tcl packages, just as a library file contains
multiple files.
.PP
The package file is just a regular Unix text file, editable with your
favorite text editor, containing packages of Tcl source code, where
package definitions are notated by the \fBpackage\fR keyword occurring in
column one of a line that occurs before any executable code in the input file,
followed (on the same line) by the package name, then by one or more
procedures that are defined by that package where you want the package
demand-loaded if an attempt is made to execute one of the specified package
procedures
and the procedure is not already defined.  For example, in a package source
file, the presence of the following line...
.PP
\fBpackage directory_stack pushd popd\fR
.PP
...says that the text lines following that line in the package file up
to the next package line or the end of the file is a package named
\fBdirectory_stack\fR and that an attempt to execute either \fIpushd\fR
or \fIpopd\fR when the routine is not already defined will cause the
\fBdirectory_stack\fR
portion of the package file to be demand-loaded.
.PP
The names of package library files should end in \fI.tlib\fR.
.PP
.SH PACKAGE INDEX FILES
A package library file has associated with it an index file 
called a \fB.tndx\fR file.  The \fB.tndx\fR file contains the names
of the packages in the \fB.tlib\fR file, their addresses and lengths
within the \fB.tlib\fR file and the functions that are to cause the
different packages to be autoloaded when an attempt is made to execute
them.
.PP
The first time Tcl tries to execute a procedure where the procedure doesn't
exist and isn't an autoload, Tcl will search along \fBTCLPATH\fR looking
for any files ending in \fB.tlib\fR.  For each one it finds, it checks
to see if there is a corresponding file in the same directory ending
in \fB.tndx\fR.  If the \fB.tndx\fR file doesn't exist, or if its date
of last modification is older than that of the \fB.tlib\fR file,
the \fB.tndx\fR is automatically (re)generated if possible.  If it can't
regenerate the file (most likely due to file or directory permission
problems), a warning message is issued.
.PP
.SH HELPFUL FUNCTIONS
\fBshowproc\fR will locate and display a procedure by name, demand-loading
it if necessary.
.PP
\fBautoprocs\fR will list the names of all known autoload and
autopackage procedures.
.PP
\fBpackages\fR will return a list of the names of all known packages.
