#
# dlnews - tcl program to let you download all articles in a newsgroup,
# unread ones or named ones
#
# not complete yet, but does some interesting stuff
#

global spool lib

set spool /usr1/spool/news
set lib /usr/lib/news

global activehash newsrchash

global newsgroup grouppath article_list

set activehash [create_hashtable 1000]
set newsrchash [create_hashtable 1000]

global active_entry low_article high_article

proc parse_active {} {
	global lib activehash
	print "parsing active file..."
	set activefp [open $lib/active r]

	while {[gets $activefp line]} {
		puthash $activehash [index $line 0] [range $line 1 end]
	}
	close $activefp
	print "\n"
}

proc parse_newsrc {} {
	global lib newsrchash
	print "parsing newsrc file..."
	set newsrcfp [open [glob ~/.newsrc] r]

	while {[gets $newsrcfp line]} {
		parse_newsrc_line $line
		puthash $newsrchash $newsrc_group $newsrc_data
	}
	close $newsrcfp
	print "\n"
}

echo "\nHackercorp Usenet News Download System\tv1.0\n"

proc set_grouppath {mynewsgroup} {
	global grouppath spool newsgroup
	global active_entry activehash low_article high_article

	set newsgroup $mynewsgroup

	set grouppath $spool/[join / [split . $newsgroup]]

	set active_entry [gethash $activehash $newsgroup]
	set low_article [index $active_entry 1]
	set high_article [index $active_entry 0]
	set result ""
}

proc set_bitmap_from_newsrc {} {
	global newsgroup newsrchash

	set newsrc_entry [gethash $newsrchash $newsgroup]
	if [empty $newsrc_entry] return
	foreach i [split , [index $newsrc_entry 1]] {
		set rangelist [split - $newsrc_entry]
		if {[length $rangelist] == 1} {
			set_bitmap_range $rangelist
		} else {
			set_bitmap_range [index $rangelist 0] [index $rangelist 1]
		}
	}
}

proc query_newsgroup {} {
	global newsrchash 
	while {1} {
	   print "Newsgroup: "
	   set a [gets file0]
	   if [empty $a] {exit 0}
	   set line [gethash $newsrchash $a]
	   if ![empty $line] {return $a}
	   print "That newsgroup doesn't exist.\n"
	}
}

proc query_specific_articles {} {
	global grouppath low_article high_article

	print "Article numbers range from about $low_article to $high_article\n"
	print "Enter article numbers, separated by spaces.\n"
	print "You may enter multiple lines -- enter an empty line to finish\n"
	set result ""
	set illegal_chars "*\[A-Za-z./~|!\]*"
	cd $grouppath

	while {1} {
		set a [gets file0]
		if [empty $a] {return [sort $result]}
		set nogood ""
		foreach i $a {
			if {[string match $illegal_chars $i] || ![file $i exists]} {
				set nogood [concat $nogood $i]
				continue
			}
			set result [concat $result $i]
		}
		if ![empty $nogood] {
			print [format "couldn't access %s\n" $nogood]
		}
	}
}

proc list_all_articles {} {
	global grouppath

	cd $grouppath
	sort [glob \[0-9\]*]
}

#
# return a sorted list of the unread articles in the current group
#  assumes newsgroup has been set and set_grouppath performed
#
proc list_unread_articles {} {
	global activehash newsgroup grouppath newsrchash
	global active_entry low_article high_article

	print "scanning..."
	set result ""
	init_bitmap $low_article $high_article
	set_bitmap_from_newsrc
	cd $grouppath
	foreach i [sort [glob \[0-9\]*]] {
		if ![fetch_bitmap_bit $i] {
			set result [concat $result $i]
		}
	}
	puthash $newsrchash $newsgroup [list [index [gethash $newsrchash $newsgroup] 0] [dump_bitmap]]
	print "\n"
	return [sort $result]
}

proc query_which_articles {} {
	while {1} {
		print "s=specific articles, u=unread articles, a=all articles: "
		set a [gets file0]
		if [empty $a] {
			exit 0
		}
		if [string match "\[sS\]*" $a] {
			return 0
		}
		if [string match "\[uU\]*" $a] {
			return 1
		}
		if [string match "\[aA\]*" $a] {
			return 2
		}
	}
}

proc query_newsgroup_and_articles {} {
	global newsgroup article_list

	set newsgroup [query_newsgroup]
	set_grouppath $newsgroup
	set how [query_which_articles]
	if {$how == 0} {
		set article_list [query_specific_articles]
	} 
	if {$how == 1} {
		set article_list [list_unread_articles]
	} 
	if {$how == 2} {
		set article_list [list_all_articles]
	}
	print [format "%d articles selected.\n\n" [length $article_list]]
}

proc do_archive {} {
	global article_list

	set filename /tmp/dlnews[getpid]
	set rawbytes 0

	print "building zoo archive...\n"

	set zooname_pipe [open "|zoo aI $filename" w]

	foreach i $article_list {
		incr rawbytes [file $i size]
		puts $zooname_pipe $i
	}
	close $zooname_pipe
	wait
	print "\n\n"
	print "$rawbytes bytes in selected articles\n"
	print "[file filename size] bytes in zoo archive\n"
	print "zoo archive is $filename\n"
}

proc query {} {
	global article_list newsgroup

	query_newsgroup_and_articles
}

proc run {} {
	parse_active
	parse_newsrc
}

